

#ifndef __IRR_C_RENDER_WINDOW_WIN32_H__
#define __IRR_C_RENDER_WINDOW_WIN32_H__



#include "IRenderWindow.h"
#include "IrrCompileConfig.h"

#ifdef _IRR_COMPILE_WITH_WINDOWS_DEVICE_



# define WIN32_LEAN_AND_MEAN
# if !defined(_IRR_XBOX_PLATFORM_)
#  include <windows.h>
#  include <windowsx.h>
# endif
# if !defined(GET_X_LPARAM)
#  define GET_X_LPARAM(lp) ((int)(short)LOWORD(lp))
#  define GET_Y_LPARAM(lp) ((int)(short)HIWORD(lp))
# endif

namespace irr
{

namespace video
{

//class IImage;

// TODO we should be able to create a CRenderWindowWin32  from a valid WindowId
class CRenderWindowWin32 : public IRenderWindow
{

public:
    /** Default constructor.
    //SRenderWindowConfig& config,
    If we are creating a window from nothing
    */
//    CRenderWindowWin32(
//                       WNDPROC fn,
//                       const SRenderWindowConfig& config
////                       HWND windowId,bool externalWindow,bool isNonNTWindows
//                       );

    /**
     In case window already exists
    **/
    CRenderWindowWin32(HWND windowId, bool fullscreeen,bool externalWindow);

    virtual ~CRenderWindowWin32();


    //! used internally. Move to device ?
//    static void registerClass() const;

    // TODO remove windowId
//    bool present(video::IImage* image, core::rect<s32>* src);


//    virtual core::dimension2du getBorderThickness();
//    startRendering(IVideoContext*)
//    setContext(IVideoContext*)

    //! sets the caption of the window
    virtual void setWindowCaption(const wchar_t* text)  ;

    //! returns if window is active. if not, nothing need to be drawn
    virtual bool isWindowActive() const  ;

    //! returns if window has focus
    virtual bool isWindowFocused() const  ;

    //! returns if window is minimized
    virtual bool isWindowMinimized() const  ;

    //! AdjustWindowRect
    core::dimension2d<u32> getClientSize() const;

    //! relative to upper-left point of screen (0/0)
    core::dimension2d<u32> getPosition() const;



//    core::dimension2d<u32> getInnerPosition() const;
//    core::dimension2d<u32> getInnerSize() const;

    virtual bool isFullscreen() const  ;

    //! Get the current color format of the window
    /** \return Color format of the window. */
    virtual video::ECOLOR_FORMAT getColorFormat() const ;

    //! Notifies the device, that it has been resized
    /** used internally only ? **/
    virtual bool resize(const core::dimension2du& newSize);

    //! Sets if the window should be resizable in windowed mode.
    virtual bool setResizable(bool resize=false)  ;

    virtual bool isResizable()  const;


    // bool reset
    virtual bool setFullscreen(const bool& enable);


    //! Minimizes the window.
    virtual void minimizeWindow()  ;

    //! Maximizes the window.
    virtual void maximizeWindow()  ;

    //! Restores the window size.
    virtual void restoreWindow()  ;

    virtual void closeWindow(bool forceDestroy = false) ;

    virtual void forceDestroy() ;


// TODO returns void* ?
//    HWND getWindowHandle() const { return WindowHandle;};
    void* getWindowHandle()  { return WindowHandle;};

    HWND getHWnd() const {
        return WindowHandle;
    };

protected:
    HWND WindowHandle;  //! Window Handle
    RECT OldWindowedRect;

    bool IsFullscreen;
//    bool ChangedToFullScreen;
    bool IsNonNTWindows;        //! Why that
//    bool Resized;

};



}  // video
} // irr

#endif

#endif
